/* oled_clock3
 *  
 * Arduino óra DS3231 óra modullal és SSD1306 OLED 
 * kijelzővel. Az eredeti programot a Simple Project
 * honlapon publikálták "Arduino real time clock
 * using DS1307 and SSD1306 OLED" címmel.
 * Link: https://simple-circuit.com/arduino-ds1307-ssd1306-oled/
 * Az adatstruktúrát, a paraméterek szerkesztését és az
 * RTC kezelését a jobb áttekinthetőség érdekében átírtuk,
 * de a program funkcionalitása nem változott. 
 */

#include <Wire.h>                        // Include Wire library (required for I2C devices)
#include <DS3231.h>
#include <Adafruit_GFX.h>                // Include Adafruit graphics library
#include <Adafruit_SSD1306.h>            // Include Adafruit SSD1306 OLED driver
#include "clock_params.h"
#define SCREEN_WIDTH 128                 // OLED display width, in pixels
#define SCREEN_HEIGHT 64                 // OLED display height, in pixels 
#define OLED_RESET    -1                 // Reset pin # (or -1 if sharing Arduino reset pin)

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);
DS3231 Clock;                            // Az RTC példányosítása

#define button1    A0                    // Button B1 is connected to Arduino pin A0
#define button2    A1                    // Button B2 is connected to Arduino pin A1

//---------------------------------------------------
// Setup function: hardware initialization
//---------------------------------------------------
void setup(void) {
  pinMode(button1, INPUT_PULLUP);
  pinMode(button2, INPUT_PULLUP);
  Clock.enableOscillator(true, 0, 0);
  delay(1000);

  // by default, we'll generate the high voltage from the 3.3v line internally! (neat!)
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3C 
  // Clear the display buffer.
  display.clearDisplay();
  display.display();
  display.setTextColor(WHITE, BLACK);
}

char Time[]     = "  :  :  ";
char Calendar[] = "  /  /20  ";
bool h12, pm, century;

//---------------------------------------------------
// Display date and time
//---------------------------------------------------
void DS1307_display() {
  Time[7]     = par[0].val % 10 + 48;
  Time[6]     = par[0].val / 10 + 48;
  Time[4]     = par[1].val % 10 + 48;
  Time[3]     = par[1].val / 10 + 48;
  Time[1]     = par[2].val % 10 + 48;
  Time[0]     = par[2].val / 10 + 48;
  Calendar[9] = par[6].val % 10 + 48;
  Calendar[8] = par[6].val / 10 + 48;
  Calendar[4] = par[5].val % 10 + 48;
  Calendar[3] = par[5].val / 10 + 48;
  Calendar[1] = par[4].val % 10 + 48;
  Calendar[0] = par[4].val / 10 + 48;
  draw_text(4,  18, Calendar, 2);
  draw_text(16, 42, Time, 2);
}

//---------------------------------------------------
// Draw a piece of a text line at a given position 
//---------------------------------------------------
void draw_text(byte x_pos, byte y_pos, char *text, byte text_size) {
  display.setCursor(x_pos, y_pos);
  display.setTextSize(text_size);
  display.print(text);
  display.display();
}

//---------------------------------------------------
// Display the ith parameter or blank space
//---------------------------------------------------
void display_parameter(byte i, boolean flag) {
  char text[3];
  if (flag) {
    if (i != 3) {
      text[1] = par[i].val%10 + 48;
      text[0] = par[i].val/10 + 48; 
      text[2] = 0;
      draw_text(par[i].xpos, par[i].ypos, text, 2);      
    } 
    else {
      draw_text(par[3].xpos,par[3].ypos,dayname[par[3].val],1);
    }
  }
  else {
    if (i != 3) {
      draw_text(par[i].xpos, par[i].ypos, "  ", 2);      
    } 
    else {
      draw_text(par[3].xpos,par[3].ypos,dayname[0],1);
    }    
  }
}

//---------------------------------------------------
// Wait for button press for maximum 250 ms 
//---------------------------------------------------
void blink_parameter() {
  byte j = 0;
  while (j < 10 && digitalRead(button1) && digitalRead(button2)) {
    j++;
    delay(25);
  }
}

//---------------------------------------------------
// Edit a time or date parameter 
// The B2 button increases the parameter value
// Exit when B1 is pressed again
//---------------------------------------------------
void edit(byte i) {
  while (!digitalRead(button1));                     // Wait until button B1 released
  while (true) {
    while (!digitalRead(button2)) {                  // If button B2 is pressed
      par[i].val = (par[i].val - par[i].base + 1) % par[i].lim + par[i].base;
      display_parameter(i, true);
      delay(200);                                    // Wait 200ms
    }  
    display_parameter(i, false);                     // Hide parameter
    blink_parameter();                               // Wait for max. 250 ms
    display_parameter(i, true);                      // Display parameter
    blink_parameter();                               // Wait for max. 250 ms
    if (!digitalRead(button1)) {                     // If button B1 is pressed
      return;                                        // Return parameter value and exit
    }
  }
}

//---------------------------------------------------
// Loop function: handles time setting when B1 is
// pressed, save/read and display time/date
//---------------------------------------------------
void loop() {
  if (!digitalRead(button1)) {           // If button B1 is pressed
    //-- Scan and edit parameters if needed --------
    edit(3);                             // Edit day of week
    edit(4);                             // Edit date
    edit(5);                             // Edit month
    edit(6);                             // Edit year
    edit(2);                             // Edit hours
    edit(1);                             // Edit minutes

    //-- Save time/date to the RTC module after modifications --- 
    Clock.setSecond(0);                  // Set Seconds to 0 and enable oscillator
    par[0].val = 0;
    Clock.setMinute(par[1].val);
    Clock.setHour(par[2].val);
    Clock.setDoW(par[3].val);
    Clock.setDate(par[4].val);
    Clock.setMonth(par[5].val);
    Clock.setYear(par[6].val);
    Clock.setClockMode(0);               // 24h mode
    delay(200);                          // Wait 200ms
  }

  //-- Read time/date from the RTC module --------- 
  par[0].val = Clock.getSecond();
  par[1].val = Clock.getMinute();
  par[2].val = Clock.getHour(h12, pm);
  // if (h12 && pm) par[2].val = (par[2].val+12)%24;
  par[3].val = Clock.getDoW();
  par[4].val = Clock.getDate();
  par[5].val = Clock.getMonth(century);
  par[6].val = Clock.getYear();
  display_parameter(3, true);                   // Display day of week
  DS1307_display();                             // Diaplay time & calendar
  delay(50);                                    // Wait 50ms
}
// End of code.
